﻿/** 
 * ---------------------------------------------------------------------------
 * Vine Box
 * ---------------------------------------------------------------------------
 * Rev.1  XXXX/XX/XX
 *   変更履歴
 */
using System;
using System.Collections;
using Server.Gumps;
using Server.Multis;
using Server.Network;
using Server.Engines.Plants;
using System.Collections.Generic;
using Server.ContextMenus;
using Server.Prompts;

namespace Server.Items
{
    class VineBox : Item, ISecurable, IDyable, ITokunoDyable

    {
        /** つる性植物列挙 */
        public enum VineType
        {
            Vine_1_South, // つるＡ（南）
            Vine_1_East,  // つるＡ（東）
            Vine_2_South, // つるＢ（南）
            Vine_2_East,  // つるＢ（東）
            Vine_3_South, // つるＣ（南）
            Vine_3_East,  // つるＣ（東）
            Vine_4_South, // つるＤ（右）
            Vine_4_East,  // つるＤ（左）
        }

        private SecureLevel m_Level;
        [CommandProperty(AccessLevel.GameMaster)]
        public SecureLevel Level
        {
            get
            {
                return this.m_Level;
            }
            set
            {
                this.m_Level = value;
            }
        }

        /** デバッグフラグ */
        private static bool isDebug = false;
        /** 最大登録数 */
        private static readonly int MAX_COUNT = 9999;
        /** アイテムデータ */
        private VineBoxData oVineBoxData;
        /** アイテム名称 */
        private String m_boxName = "";

        [CommandProperty(AccessLevel.GameMaster)]
        public string binderName
        {
            get
            {
                return this.m_boxName;
            }
            set
            {
                this.m_boxName = value;
                this.InvalidateProperties();
            }
        }

        /** 現在表示しているメニューページ番号
         * 0：何も開いていない
         * 1：つる性植物を開いている
         */
        private int nowMenuPage = 0;

        /** 現在表示しているメインページ番号
         *  0：何も開いていない
         *  ----旧植物----(8種)
         *  1：つるＡ（南） ID : 0xCEB
         *  2：つるＡ（東） ID : 0xCEF
         *  3：つるＢ（南） ID : 0xCEC
         *  4：つるＢ（東） ID : 0xCF0
         *  5：つるＣ（南） ID : 0xCED
         *  6：つるＣ（東） ID : 0xCF1
         *  7：つるＤ（右） ID : 0xCEE
         *  8：つるＤ（左） ID : 0xCF2
         */
        private int nowPage = 0;

        [Constructable]
        public VineBox()
                : base(0x232A)
        {
            this.Weight = 1.0;
            this.LootType = LootType.Blessed;
            this.Hue = 0x480;
            this.Name = "おっさんの憂鬱";
            this.m_Level = SecureLevel.Owner;

            this.oVineBoxData = new VineBoxData();
        }
        public VineBox(Serial serial)
            : base(serial)
        {
        }
        public override void GetContextMenuEntries(Mobile from, List<ContextMenuEntry> list)
        {
            base.GetContextMenuEntries(from, list);

            if (from.CheckAlive() && this.IsChildOf(from.Backpack))
                list.Add(new NameBinderEntry(from, this));

            SetSecureLevelEntry.AddTo(from, this, list);
        }

        public override void OnDoubleClick(Mobile from)
        {
            if (from.FindGump(typeof(VineBox.InternalGump)) == null)
                from.SendGump(new InternalGump(this, 0));
        }
        public override bool OnDragDrop(Mobile from, Item dropped)
        {
            // 登録数MAX(以上はありえないけど念のため)
            if (oVineBoxData.TotalCount >= MAX_COUNT)
            {
                from.SendMessage("これ以上追加できません。");
                return false;
            }
            // ドロップされたアイテムがつる性植物ではない
            Boolean flg = false;
            foreach (VineType value in Enum.GetValues(typeof(VineType)))
            {
                if (dropped.ItemID == VineBoxInfo.GetInfo(value).ItemID)
                {
                    flg = true;
                    oVineBoxData.countPlus(value);
                    break;
                }
            }
            if(!flg)
            {
                from.SendMessage("そういうイジワルはしちゃあかんです！！");
                return false;
            }
            // Rev.1(Add S) --------------------------------------------------
            // プロパティを更新
            this.InvalidateProperties();
            // バインダーのガンプを表示していたら閉じる
            from.CloseGump(typeof(InternalGump));
            // バインダーを開く
            from.SendGump(new InternalGump(this, 1));
            // Rev.1(Add E) --------------------------------------------------
            from.SendMessage("つる性植物を追加しました！");
            return true;
        }

        /**
         * つる性植物データを返します。
         * 
         */
        public VineBoxData getVineBoxData()
        {
            return this.oVineBoxData;
        }
        /**
         * 現在のメニューページを返します。
         */
        public int getNowMenuPage()
        {
            return nowMenuPage;
        }
        /**
         * 指定したのメニューページへ進みます
         */
        public void gotoMenuPage(int menuPage)
        {
            this.nowMenuPage = menuPage;
        }
        /**
         * 現在のメインページを返します。
         */
        public int getNowPage()
        {
            return nowPage;
        }
        /**
         * 指定したのメインページへ進みます
         */
        public void gotoPage(int page)
        {
            this.nowPage = page;
        }
        /**
         * 植物を復元します。
         */
        public void releseItem(Mobile mob, VineType vt)
        {
            if (VineBox.isDebug)
            {
            }
            // 重量判定
            if ((mob.MaxWeight - mob.TotalWeight) < 50)
            {
                mob.SendMessage("バックパックの重量に余裕がありません。");
                return;
            }
            // 所持数判定
            if (mob.Backpack.TotalItems > 124)
            {
                mob.SendMessage("バックパックの空きに余裕がありません。");
                return;
            }
            if (oVineBoxData.getItemCount(vt) > 0)
            {
                oVineBoxData.countMinus(vt);
            }
            else
            {
                mob.SendMessage("該当のつる性植物がありません。");
                return;
            }
            Item item = null;
            // 存在する場合はバックパックに復元
            switch (vt)
            {
                case VineType.Vine_1_South:
                    item = new Vines(0);
                    break;
                case VineType.Vine_2_South:
                    item = new Vines(1);
                    break;
                case VineType.Vine_3_South:
                    item = new Vines(2);
                    break;
                case VineType.Vine_4_South:
                    item = new Vines(3);
                    break;
                case VineType.Vine_1_East:
                    item = new Vines(4);
                    break;
                case VineType.Vine_2_East:
                    item = new Vines(5);
                    break;
                case VineType.Vine_3_East:
                    item = new Vines(6);
                    break;
                case VineType.Vine_4_East:
                    item = new Vines(7);
                    break;

            }
            mob.SendMessage("つる性植物を取り出しました。");
            // Rev.1(Add S) --------------------------------------------------
            // プロパティを更新
            this.InvalidateProperties();
            // Rev.1(Add E) --------------------------------------------------
            mob.Backpack.DropItem(item);
        }
        public override void GetProperties(ObjectPropertyList list)
        {
            base.GetProperties(list);
            // Rev.1(Add S) --------------------------------------------------
            list.Add(1062344, this.getVineBoxData().TotalCount + "/" + MAX_COUNT);
            // Rev.1(Add E) --------------------------------------------------
            if (this.m_boxName != null && this.m_boxName.Length > 0)
                list.Add("名前 : " + this.m_boxName); // Book Name: ~1_val~

        }

        private class InternalGump : Gump
        {
            private int mainPageX = 260;
            private int nKind = 0;

            // リストから割り出した必要ページ数
            private int page = 0;

            // 1ページあたりの最大表示数
            private static readonly int PAGE_MAX_DISP = 10;
            // 半ページあたりの最大表示数
            private static readonly int HALF_PAGE_MAX_DISP = 5;
            // 最大ページ数
            private static readonly int PAGE_MAX = 3;


            private VineBox box;
            public InternalGump(VineBox box, int nKind)
                : base(150, 50)
            {
                this.box = box;
                this.AddPage(0);
                // 背景
                this.AddImageTiled(26, 26, 560, 480, 0x0DB0);

                // 左上
                this.AddImage(0, 0, 0x0DAC);                   // width26 height26
                // 上
                this.AddImageTiled(26, 0, 560, 26, 0x0DAD);
                // 右上
                this.AddImage(560, 0, 0x0DAE);        // width26 height26
                // 左
                this.AddImageTiled(0, 26, 26, 480 - 26, 0x0DAF);
                // 右
                this.AddImageTiled(560, 26, 26, 480 - 26, 0x0DB1);

                // タイトルから少し下に横線
                this.AddImageTiled(10, 80, 560, 26, 0x0DB3);

                // メニュー区切り
                this.AddImageTiled(200, 100, 26, 400, 0x0DB1);

                // 下
                this.AddImageTiled(26, 480, 560 - 26, 26, 0x0DB3);
                // 左下
                this.AddImage(0, 480, 0x0DB2);
                // 右下
                this.AddImage(560, 480, 0x0DB4);

                this.AddHtml(50, 30, 120, 20, "<BASEFONT COLOR =#006400><CENTER><BIG>" + this.box.Name, false, false);
                this.AddItem(68, 50, 0x0C98);
                this.AddItem(530, 55, 0x11CB);
                this.AddButton(230, 30, 0x0837, 0x0838, 101, GumpButtonType.Reply, 0);
                this.AddHtml(230 + 25, 27, 200, 20, "つる性植物", false, false);
                
                if (this.box.getNowMenuPage() == 1)
                {
                    foreach (VineType value in Enum.GetValues(typeof(VineType)))
                    {
                        this.AddButton(30, 85 + (20 * ((int)value + 1)), 0x0837, 0x0838, ((int)value + 1), GumpButtonType.Reply, 0);
                        // 現在登録されている該当植物の総数を取得
                        int count = box.getVineBoxData().getItemCount(value);
                        String text = VineBoxInfo.GetInfo(value).ItemName + "(" + count + ")";
                        this.AddHtml(55, 82 + (20 * ((int)value + 1)), 200, 20, text, false, false);
                    }
                }
                // 
                if (this.box.getNowMenuPage() != 0 && this.box.getNowPage() != 0)
                {
                     int offset = 25;
                    this.AddImageTiled(270 + offset, 105, 200, 200, 0x248A);

                    this.AddImageTiled(270 + offset, 100, 200, 4, 0x248D);
                    this.AddImageTiled(270 + offset, 305, 200, 4, 0x248D);
                    this.AddImageTiled(270 + offset, 100, 4, 200, 0x248B);
                    this.AddImageTiled(470 + offset, 100, 4, 200, 0x248B);

                    // 選択された植物のタイプを取得
                    VineBoxInfo info = VineBoxInfo.GetInfo(this.box.getNowPage() - 1);
                    Rectangle2D b = ItemBounds.Table[info.ItemID];

                    this.AddItem((370 + offset) - b.Width / 2 - b.X, 215 - b.Height / 2 - b.Y, info.ItemID);

                    this.AddButton(230, 295 + 20, 0x5787, 0x5787, 200, GumpButtonType.Reply, 0);
                    this.AddHtml(250, 292 + 20, 200, 20, "このアイテムを取り出します。", false, false);
                }

            }

            public override void OnResponse(NetState sender, RelayInfo info)
            {
                int button = info.ButtonID;
                Mobile mob = sender.Mobile;

                switch (button)
                {
                    case 101:
                        this.box.gotoMenuPage(1);
                        this.box.gotoPage(0);
                        mob.CloseGump(typeof(InternalGump));
                        mob.SendGump(new InternalGump(this.box, 1));
                        break;
                    case 102:
                        this.box.gotoMenuPage(2);
                        this.box.gotoPage(0);
                        mob.CloseGump(typeof(InternalGump));
                        mob.SendGump(new InternalGump(this.box, 1));
                        break;
                    case 103:
                        this.box.gotoMenuPage(3);
                        this.box.gotoPage(0);
                        mob.CloseGump(typeof(InternalGump));
                        mob.SendGump(new InternalGump(this.box, 1));
                        break;
                    default:
                        if (this.box.getNowMenuPage() == 1 && button > 0 && button < 18)
                        {
                            this.box.gotoPage(button);
                            mob.CloseGump(typeof(InternalGump));
                            mob.SendGump(new InternalGump(this.box, 1));
                        }
                       else if (this.box.getNowMenuPage() == 1 && button >= 200)
                        {
                            VineType vt = VineBoxInfo.GetInfo(this.box.getNowPage() - 1).VineType;;
                            this.box.releseItem(mob, vt);
                            mob.CloseGump(typeof(InternalGump));
                            mob.SendGump(new InternalGump(this.box, 1));
                        }
                        break;
                }
            }
        }

        private class NameBinderEntry : ContextMenuEntry
        {
            private readonly Mobile m_From;
            private readonly VineBox m_Binder;
            public NameBinderEntry(Mobile from, VineBox binder)
                : base(6216)
            {
                this.m_From = from;
                this.m_Binder = binder;
            }

            public override void OnClick()
            {
                if (this.m_From.CheckAlive() && this.m_Binder.IsChildOf(this.m_From.Backpack))
                {
                    this.m_From.Prompt = new NameBinderPrompt(this.m_Binder);
                    this.m_From.SendMessage("新しい名前を付けてください。"); // Type in the new name of the book:
                }
            }
        }
        private class NameBinderPrompt : Prompt
        {
            private readonly VineBox m_Binder;
            public NameBinderPrompt(VineBox binder)
            {
                this.m_Binder = binder;
            }

            public override void OnResponse(Mobile from, string text)
            {
                if (text.Length > 40)
                    text = text.Substring(0, 40);

                if (from.CheckAlive() && this.m_Binder.IsChildOf(from.Backpack))
                {
                    this.m_Binder.binderName = Utility.FixHtml(text.Trim());

                    from.SendMessage("名前を変更しました。"); // The bulk order book's name has been changed.
                }
            }

            public override void OnCancel(Mobile from)
            {
            }
        }

        private class VineBoxInfo
        {
            private static VineBoxInfo[] dataTable =
            {
                new VineBoxInfo(0xCEB, VineType.Vine_1_South, "つるＡ（南）"),
                new VineBoxInfo(0xCEF, VineType.Vine_1_East , "つるＡ（東）"),
                new VineBoxInfo(0xCEC, VineType.Vine_2_South, "つるＢ（南）"),
                new VineBoxInfo(0xCF0, VineType.Vine_2_East , "つるＢ（東）"),
                new VineBoxInfo(0xCED, VineType.Vine_3_South, "つるＣ（南）"),
                new VineBoxInfo(0xCF1, VineType.Vine_3_East , "つるＣ（東）"),
                new VineBoxInfo(0xCEE, VineType.Vine_4_South, "つるＤ（右）"),
                new VineBoxInfo(0xCF2, VineType.Vine_4_East , "つるＤ（左）"),
            };

            private int itemID = 0;
            private VineType vineType;
            private String itemName = "";

            private VineBoxInfo(int itemID, VineType vineType,String itemName )
            {
                this.itemID = itemID;
                this.vineType = vineType;
                this.itemName = itemName;
            }

            public static VineBoxInfo GetInfo(VineType vineType)
            {
                int index = (int)vineType;

                if (index >= 0 && index < dataTable.Length)
                    return dataTable[index];
                else
                    return dataTable[0];
            }

            public static VineBoxInfo GetInfo(int index)
            {
                if (index >= 0 && index < dataTable.Length)
                    return dataTable[index];
                else
                    return dataTable[0];
            }

            public int ItemID { get { return itemID; } }
            public VineType VineType { get { return vineType; } }
            public String ItemName { get { return itemName; } }
        }

        public class VineBoxData
        {
            private int nTotalCount = 0;
            private int[] nItemCount = new int[Enum.GetNames(typeof(VineType)).Length];

            public VineBoxData()
            {
                nTotalCount = 0;
                foreach (VineType value in Enum.GetValues(typeof(VineType)))
                {
                    nItemCount[(int)value] = 0;
                }
            }

            public VineBoxData(GenericReader reader)
            {
                int version = reader.ReadInt();
                nTotalCount = reader.ReadInt();
                foreach (VineType value in Enum.GetValues(typeof(VineType)))
                {
                    nItemCount[(int)value] = reader.ReadInt();
                }
            }

            public int TotalCount { get { return nTotalCount; } set { this.nTotalCount = value; } }

            public void countPlus(VineType vt)
            {
                this.nItemCount[(int)vt]++;
                int count = 0;
                foreach (VineType value in Enum.GetValues(typeof(VineType)))
                {
                    count += nItemCount[(int)value];
                }
                this.nTotalCount = count;
            }

            public void countMinus(VineType vt)
            {
                this.nItemCount[(int)vt]--;
                int count = 0;
                foreach (VineType value in Enum.GetValues(typeof(VineType)))
                {
                    count += nItemCount[(int)value];
                }
                this.nTotalCount = count;
            }

            public void setItemCount(VineType vt, int count)
            {
                this.nItemCount[(int)vt] = count;
            }

            public int getItemCount(VineType vt)
            {
                return this.nItemCount[(int)vt];
            }

            public void Serialize(GenericWriter writer)
            {
                writer.Write(0); // version
                writer.Write(nTotalCount);
                foreach (VineType value in Enum.GetValues(typeof(VineType)))
                {
                    writer.Write(nItemCount[(int)value]);
                }
            }
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);

            writer.Write((int)0); // version
            writer.Write((int)m_Level); // セキュアレベル
            object obj = oVineBoxData;
            ((VineBoxData)obj).Serialize(writer);
             writer.Write(this.binderName); // 名前
            writer.Write(this.Hue); // 色
        }


        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
            this.m_Level = (SecureLevel)reader.ReadInt();
            this.oVineBoxData = new VineBoxData(reader);
            this.binderName = reader.ReadString();
            this.Hue = reader.ReadInt();
        }

        public bool Dye(Mobile from, DyeTub sender)
        {
            if (this.Deleted)
                return false;

            this.Hue = sender.DyedHue;

            return true;
        }
    }
}
